rm(list=ls())
library(hdm)
#library(hdmrw) ### use implementation in hdm
library(mvtnorm)
library(knockoff)
library(foreach)

# for parallelization
library(parallel)
library(doParallel)

# for na handling
library(tidyverse)

wspace = "## PASTE YOUR WORKING SPACE HERE ###"
setwd(wspace)

date <- Sys.Date()
date

sessionInfo()

source(paste0(wspace, "/DGP_s6.R"))
#source("DGP_s6.R")

R = 2000

# correlation for Sigma 
# setting: "medium"
rho = 0.8

# Setting for x-dependent choice of lambda with rlasso (set TRUE in last version)
X.dep = TRUE

# FDR level for knockoffs (to be specified before)
gamma_knockoff = 0.1 

# Significance level alpha
alpha = 0.1
n = 200
p = 60

# load results, update path

### *to be adjusted* ###


# load results, update path
load(paste0(wspace, "/results/2020-01-10_results_medium_cor_n_200_p_60_R_5000.RData"))

### *to be adjusted* ###


# All Rejections

## p_adjust
rej_RW = vapply(res, function(x) sum(x$pvals_RW <= alpha), double(1L))
rej_BH = vapply(res, function(x) sum(x$pvals_BH <= alpha), double(1L))
rej_bonf = vapply(res, function(x) sum(x$pvals_bonf <= alpha), double(1L))
rej_holm = vapply(res, function(x) sum(x$pvals_holm <= alpha), double(1L))
rej_none = vapply(res, function(x) sum(x$pvals_none <= alpha), double(1L))

# knockoff
rej_knockoff_default <- vapply(res, function(x) sum(x$knockoff_default$selected != 0), double(1L))
rej_knockoff_fixed <- vapply(res, function(x) sum(x$knockoff_fixed$selected != 0), double(1L))
rej_knockoff_gaussian <- vapply(res, function(x) sum(x$knockoff_gaussian$selected != 0), double(1L))

# number of rejections
avgrej_none = mean(rej_none, na.rm = T)
avgrej_BH = mean(rej_BH, na.rm = T)
avgrej_bonf = mean(rej_bonf, na.rm = T)
avgrej_holm = mean(rej_holm, na.rm = T)
avgrej_RW = mean(rej_RW, na.rm = T)

avgrej_knockoff_default = mean(rej_knockoff_default, na.rm = T)
avgrej_knockoff_fixed = mean(rej_knockoff_fixed, na.rm = T)
avgrej_knockoff_gaussian = mean(rej_knockoff_gaussian, na.rm = T)

# Only repetitions with non-zero output
avgrej2_knockoff_default = mean(rej_knockoff_default[rej_knockoff_default != 0], na.rm = T)
avgrej2_knockoff_fixed = mean(rej_knockoff_fixed[rej_knockoff_fixed != 0], na.rm = T)
avgrej2_knockoff_gaussian = mean(rej_knockoff_gaussian[rej_knockoff_gaussian != 0], na.rm = T)


# number of correct rejections

# beta_0 (identical in all repetitions)
betas <- res[[1]]$beta_0
names(betas) <- paste0("Var", 1:length(betas))

avgcorrej_RW = mean(vapply(res, function(x) sum(x$pvals_RW[betas!=0] <= alpha), double(1L)), na.rm = T)
avgcorrej_BH = mean(vapply(res, function(x) sum(x$pvals_BH[betas!=0] <= alpha), double(1L)), na.rm = T)
avgcorrej_bonf = mean(vapply(res, function(x) sum(x$pvals_bonf[betas!=0] <= alpha), double(1L)), na.rm = T)
avgcorrej_holm = mean(vapply(res, function(x) sum(x$pvals_holm[betas!=0] <= alpha), double(1L)), na.rm = T)
avgcorrej_none = mean(vapply(res, function(x) sum(x$pvals_none[betas!=0] <= alpha), double(1L)), na.rm = T)

sdcorrej_RW = sd(vapply(res, function(x) sum(x$pvals_RW[betas!=0] <= alpha), double(1L)), na.rm = T)
sdcorrej_BH = sd(vapply(res, function(x) sum(x$pvals_BH[betas!=0] <= alpha), double(1L)), na.rm = T)
sdcorrej_bonf = sd(vapply(res, function(x) sum(x$pvals_bonf[betas!=0] <= alpha), double(1L)), na.rm = T)
sdcorrej_holm = sd(vapply(res, function(x) sum(x$pvals_holm[betas!=0] <= alpha), double(1L)), na.rm = T)
sdcorrej_none = sd(vapply(res, function(x) sum(x$pvals_none[betas!=0] <= alpha), double(1L)), na.rm = T)

# knockoffs (via names)
# all repetitions
avgcorrej_knockoff_default = mean(vapply(res, function(x)  sum(names(x$knockoff_default$selected) %in% names(betas[betas!=0])), double(1L)), na.rm = T)
avgcorrej_knockoff_fixed = mean(vapply(res, function(x)  sum(names(x$knockoff_fixed$selected) %in% names(betas[betas!=0])), double(1L)), na.rm = T)
avgcorrej_knockoff_gaussian = mean(vapply(res, function(x)  sum(names(x$knockoff_gaussian$selected) %in% names(betas[betas!=0])), double(1L)), na.rm = T)

sdcorrej_knockoff_default = sd(vapply(res, function(x)  sum(names(x$knockoff_default$selected) %in% names(betas[betas!=0])), double(1L)), na.rm = T)
sdcorrej_knockoff_fixed = sd(vapply(res, function(x)  sum(names(x$knockoff_fixed$selected) %in% names(betas[betas!=0])), double(1L)), na.rm = T)
sdcorrej_knockoff_gaussian = sd(vapply(res, function(x)  sum(names(x$knockoff_gaussian$selected) %in% names(betas[betas!=0])), double(1L)), na.rm = T)

# only rejections with non-zero selections
avgcorrej2_knockoff_default = mean(vapply(res, function(x)  sum(names(x$knockoff_default$selected) %in% names(betas[betas!=0])), double(1L))[rej_knockoff_default != 0], na.rm = T)
avgcorrej2_knockoff_fixed = mean(vapply(res, function(x)  sum(names(x$knockoff_fixed$selected) %in% names(betas[betas!=0])), double(1L))[rej_knockoff_fixed != 0], na.rm = T)
avgcorrej2_knockoff_gaussian = mean(vapply(res, function(x)  sum(names(x$knockoff_gaussian$selected) %in% names(betas[betas!=0])), double(1L))[rej_knockoff_gaussian != 0], na.rm = T)

sdcorrej2_knockoff_default = sd(vapply(res, function(x)  sum(names(x$knockoff_default$selected) %in% names(betas[betas!=0])), double(1L))[rej_knockoff_default != 0], na.rm = T)
sdcorrej2_knockoff_fixed = sd(vapply(res, function(x)  sum(names(x$knockoff_fixed$selected) %in% names(betas[betas!=0])), double(1L))[rej_knockoff_fixed != 0], na.rm = T)
sdcorrej2_knockoff_gaussian = sd(vapply(res, function(x)  sum(names(x$knockoff_gaussian$selected) %in% names(betas[betas!=0])), double(1L))[rej_knockoff_gaussian != 0], na.rm = T)


# number of incorrect rejections
avgincorrej_RW = mean(vapply(res, function(x) sum(x$pvals_RW[betas==0] <= alpha), double(1L)), na.rm = T)
avgincorrej_BH = mean(vapply(res, function(x) sum(x$pvals_BH[betas==0] <= alpha), double(1L)), na.rm = T)
avgincorrej_bonf = mean(vapply(res, function(x) sum(x$pvals_bonf[betas==0] <= alpha), double(1L)), na.rm = T)
avgincorrej_holm = mean(vapply(res, function(x) sum(x$pvals_holm[betas==0] <= alpha), double(1L)), na.rm = T)
avgincorrej_none = mean(vapply(res, function(x) sum(x$pvals_none[betas==0] <= alpha), double(1L)), na.rm = T)

sdincorrej_RW = sd(vapply(res, function(x) sum(x$pvals_RW[betas==0] <= alpha), double(1L)), na.rm = T)
sdincorrej_BH = sd(vapply(res, function(x) sum(x$pvals_BH[betas==0] <= alpha), double(1L)), na.rm = T)
sdincorrej_bonf = sd(vapply(res, function(x) sum(x$pvals_bonf[betas==0] <= alpha), double(1L)), na.rm = T)
sdincorrej_holm = sd(vapply(res, function(x) sum(x$pvals_holm[betas==0] <= alpha), double(1L)), na.rm = T)
sdincorrej_none = sd(vapply(res, function(x) sum(x$pvals_none[betas==0] <= alpha), double(1L)), na.rm = T)

# knockoffs (via names)
# all repetitions
avgincorrej_knockoff_default = mean(vapply(res, function(x)  sum(names(x$knockoff_default$selected) %in% names(betas[betas==0])), double(1L)), na.rm = T)
avgincorrej_knockoff_fixed = mean(vapply(res, function(x)  sum(names(x$knockoff_fixed$selected) %in% names(betas[betas==0])), double(1L)), na.rm = T)
avgincorrej_knockoff_gaussian = mean(vapply(res, function(x)  sum(names(x$knockoff_gaussian$selected) %in% names(betas[betas==0])), double(1L)), na.rm = T)

sdincorrej_knockoff_default = sd(vapply(res, function(x)  sum(names(x$knockoff_default$selected) %in% names(betas[betas==0])), double(1L)), na.rm = T)
sdincorrej_knockoff_fixed = sd(vapply(res, function(x)  sum(names(x$knockoff_fixed$selected) %in% names(betas[betas==0])), double(1L)), na.rm = T)
sdincorrej_knockoff_gaussian = sd(vapply(res, function(x)  sum(names(x$knockoff_gaussian$selected) %in% names(betas[betas==0])), double(1L)), na.rm = T)

# only repetitions with non-zero selection
avgincorrej2_knockoff_default = mean(vapply(res, function(x)  sum(names(x$knockoff_default$selected) %in% names(betas[betas==0])), double(1L))[rej_knockoff_default != 0], na.rm = T)
avgincorrej2_knockoff_fixed = mean(vapply(res, function(x)  sum(names(x$knockoff_fixed$selected) %in% names(betas[betas==0])), double(1L))[rej_knockoff_fixed != 0], na.rm = T)
avgincorrej2_knockoff_gaussian = mean(vapply(res, function(x)  sum(names(x$knockoff_gaussian$selected) %in% names(betas[betas==0])), double(1L))[rej_knockoff_gaussian != 0], na.rm = T)

sdincorrej2_knockoff_default = sd(vapply(res, function(x)  sum(names(x$knockoff_default$selected) %in% names(betas[betas==0])), double(1L))[rej_knockoff_default != 0], na.rm = T)
sdincorrej2_knockoff_fixed = sd(vapply(res, function(x)  sum(names(x$knockoff_fixed$selected) %in% names(betas[betas==0])), double(1L))[rej_knockoff_fixed != 0], na.rm = T)
sdincorrej2_knockoff_gaussian = sd(vapply(res, function(x)  sum(names(x$knockoff_gaussian$selected) %in% names(betas[betas==0])), double(1L))[rej_knockoff_gaussian != 0], na.rm = T)


# FWER
FWER_RW = mean(vapply(res, function(x) any(x$pvals_RW[betas==0] <= alpha), double(1L)), na.rm = T)
FWER_BH = mean(vapply(res, function(x) any(x$pvals_BH[betas==0] <= alpha), double(1L)), na.rm = T)
FWER_bonf = mean(vapply(res, function(x) any(x$pvals_bonf[betas==0] <= alpha), double(1L)), na.rm = T)
FWER_holm = mean(vapply(res, function(x) any(x$pvals_holm[betas==0] <= alpha), double(1L)), na.rm = T)
FWER_none = mean(vapply(res, function(x) any(x$pvals_none[betas==0] <= alpha), double(1L)), na.rm = T)

# all repetitions 
FWER_knockoff_default = mean(vapply(res, function(x)  any(names(x$knockoff_default$selected) %in% names(betas[betas==0])), double(1L)), na.rm = T)
FWER_knockoff_fixed = mean(vapply(res, function(x)  any(names(x$knockoff_fixed$selected) %in% names(betas[betas==0])), double(1L)), na.rm = T)
FWER_knockoff_gaussian = mean(vapply(res, function(x)  any(names(x$knockoff_gaussian$selected) %in% names(betas[betas==0])), double(1L)), na.rm = T)

# only reptitions with non-zero selection
FWER2_knockoff_default = mean(vapply(res, function(x)  any(names(x$knockoff_default$selected) %in% names(betas[betas==0])), double(1L))[rej_knockoff_default != 0], na.rm = T)
FWER2_knockoff_fixed = mean(vapply(res, function(x)  any(names(x$knockoff_fixed$selected) %in% names(betas[betas==0])), double(1L))[rej_knockoff_fixed != 0], na.rm = T)
FWER2_knockoff_gaussian = mean(vapply(res, function(x)  any(names(x$knockoff_gaussian$selected) %in% names(betas[betas==0])), double(1L))[rej_knockoff_gaussian != 0], na.rm = T)

# FDR: Proportion of false among all rejections
FDR_RW = mean( vapply(res, function(x) sum(x$pvals_RW[betas==0] <= alpha), double(1L))/vapply(res, function(x) sum(x$pvals_RW <= alpha), double(1L)), na.rm = T)
FDR_BH = mean( vapply(res, function(x) sum(x$pvals_BH[betas==0] <= alpha), double(1L))/vapply(res, function(x) sum(x$pvals_BH <= alpha), double(1L)), na.rm = T)
FDR_bonf = mean( vapply(res, function(x) sum(x$pvals_bonf[betas==0] <= alpha), double(1L))/vapply(res, function(x) sum(x$pvals_bonf <= alpha), double(1L)), na.rm = T)
FDR_holm = mean( vapply(res, function(x) sum(x$pvals_holm[betas==0] <= alpha), double(1L))/vapply(res, function(x) sum(x$pvals_holm <= alpha), double(1L)), na.rm = T)
FDR_none = mean( vapply(res, function(x) sum(x$pvals_none[betas==0] <= alpha), double(1L))/vapply(res, function(x) sum(x$pvals_none <= alpha), double(1L)), na.rm = T)

# all repetitions (use reps with zero-selection as zero rejection rounds)
FDR_knockoff_default =  mean(replace_na(vapply(res, function(x)  sum(names(x$knockoff_default$selected) %in% names(betas[betas==0])), double(1L)) / rej_knockoff_default, 0), na.rm = T)
FDR_knockoff_fixed = mean(replace_na(vapply(res, function(x)  sum(names(x$knockoff_fixed$selected) %in% names(betas[betas==0])), double(1L)) / rej_knockoff_fixed, 0), na.rm = T)
FDR_knockoff_gaussian =  mean(replace_na(vapply(res, function(x)  sum(names(x$knockoff_gaussian$selected) %in% names(betas[betas==0])), double(1L)) / rej_knockoff_gaussian, 0), na.rm = T)

# only repetitions with non-zero selection
FDR2_knockoff_default = 
 mean(vapply(res, function(x)  sum(names(x$knockoff_default$selected) %in% names(betas[betas==0])), double(1L))[ rej_knockoff_default != 0] / rej_knockoff_default[ rej_knockoff_default != 0], na.rm = T)
FDR2_knockoff_fixed = 
 mean(vapply(res, function(x)  sum(names(x$knockoff_fixed$selected) %in% names(betas[betas==0])), double(1L)) / rej_knockoff_fixed, na.rm = T)
FDR2_knockoff_gaussian = 
 mean(vapply(res, function(x)  sum(names(x$knockoff_gaussian$selected) %in% names(betas[betas==0])), double(1L)) / rej_knockoff_gaussian, na.rm = T)

# compute MSE for coefficients (rlassoEffects vs. knockoffs)




#### Result table
results <- matrix( NA, ncol = 8, nrow = 12, byrow = FALSE)

# avg correct rejections
results[1, ] <- c(avgcorrej_none, avgcorrej_BH, avgcorrej_bonf, avgcorrej_holm, avgcorrej_RW, avgcorrej_knockoff_default, avgcorrej_knockoff_fixed, avgcorrej_knockoff_gaussian)

# sd correct rejection
results[2, ] <- c(sdcorrej_none, sdcorrej_BH, sdcorrej_bonf, sdcorrej_holm, sdcorrej_RW, sdcorrej_knockoff_default, sdcorrej_knockoff_fixed, sdcorrej_knockoff_gaussian)

# avg correct rejections (knockoff only with non-zero selection)
results[3, 6:8] <- c(avgcorrej2_knockoff_default, avgcorrej2_knockoff_fixed, avgcorrej2_knockoff_gaussian)

# avg correct rejections (knockoff only with non-zero selection)
results[4, 6:8] <- c(sdcorrej2_knockoff_default, sdcorrej2_knockoff_fixed, sdcorrej2_knockoff_gaussian)

# avg incorrect rejections
results[5, ] <- c(avgincorrej_none, avgincorrej_BH, avgincorrej_bonf, avgincorrej_holm, avgincorrej_RW, avgincorrej_knockoff_default, avgincorrej_knockoff_fixed, avgincorrej_knockoff_gaussian)

# sd incorrect rejections 
results[6, ] <- c(sdincorrej_none, sdincorrej_BH, sdincorrej_bonf, sdincorrej_holm, sdincorrej_RW, sdincorrej_knockoff_default, sdincorrej_knockoff_fixed, sdincorrej_knockoff_gaussian)

# avg incorrect rejections (knockoff only with non-zero selection)
results[7, 6:8] <- c(avgincorrej2_knockoff_default, avgincorrej2_knockoff_fixed, avgincorrej2_knockoff_gaussian)

# avg incorrect rejections (knockoff only with non-zero selection)
results[8, 6:8] <- c(sdincorrej2_knockoff_default, sdincorrej2_knockoff_fixed, sdincorrej2_knockoff_gaussian)

# FWER
results[9, ] <- c(FWER_none, FWER_BH, FWER_bonf, FWER_holm, FWER_RW, FWER_knockoff_default, FWER_knockoff_fixed, FWER_knockoff_gaussian)

# FWER (knockoff only with non-zero selection)
results[10, 6:8] <- c(FWER2_knockoff_default, FWER2_knockoff_fixed, FWER2_knockoff_gaussian)

# FDR 
results[11, ] <- c(FDR_none, FDR_BH, FDR_bonf, FDR_holm, FDR_RW, FDR_knockoff_default, FDR_knockoff_fixed, FDR_knockoff_gaussian)

# FDR (knockoff only with non-zero selection)
results[12, 6:8] <- c(FDR2_knockoff_default, FDR2_knockoff_fixed, FDR2_knockoff_gaussian)

colnames(results) = c("none", "BH", "bonf", "holm", "RW", "kn.off default", "kn.off fixed", "kn.off X")
rownames(results) <- c("Correct", "(sd)", "Correct2", "(sd)", "Incorrect", "(sd)", "Incorrect2", "(sd)", "FWER", "FWER2" , "FDR", "FDR2")


save(results, file = paste0(wspace, "/results/", date, "_result_merge_medium_n_", n, "_p_", p, "_R_", R, ".RData" ))

rm(list=ls())
library(hdm)
#library(hdmrw) ### use implementation in hdm
library(mvtnorm)
library(knockoff)
library(foreach)

# for parallelization
library(parallel)
library(doParallel)

# for na handling
library(tidyverse)

wspace = "/work/fwnv625/rjournal/simulation_revised"
setwd(wspace)

date <- Sys.Date()
date

sessionInfo()

source(paste0(wspace, "/DGP_s6.R"))
#source("DGP_s6.R")

R = 1000

# correlation for Sigma 
# setting: "medium"
rho = 0.8

# Setting for x-dependent choice of lambda with rlasso (set TRUE in last version)
X.dep = TRUE

# FDR level for knockoffs (to be specified before)
gamma_knockoff = 0.1 

# Significance level alpha
alpha = 0.1
n = 200
p = 180

# load results, update path

### *to be adjusted* ###


# load results, update path
load(paste0(wspace, "/results/2020-02-03_results_medium_cor_n_200_p_180_R_1000.RData"))

### *to be adjusted* ###


# All Rejections

## p_adjust
rej_RW = vapply(res, function(x) sum(x$pvals_RW <= alpha), double(1L))
rej_BH = vapply(res, function(x) sum(x$pvals_BH <= alpha), double(1L))
rej_bonf = vapply(res, function(x) sum(x$pvals_bonf <= alpha), double(1L))
rej_holm = vapply(res, function(x) sum(x$pvals_holm <= alpha), double(1L))
rej_none = vapply(res, function(x) sum(x$pvals_none <= alpha), double(1L))

# knockoff
rej_knockoff_default <- vapply(res, function(x) sum(x$knockoff_default$selected != 0), double(1L))
rej_knockoff_fixed <- vapply(res, function(x) sum(x$knockoff_fixed$selected != 0), double(1L))
rej_knockoff_gaussian <- vapply(res, function(x) sum(x$knockoff_gaussian$selected != 0), double(1L))

# number of rejections
avgrej_none = mean(rej_none, na.rm = T)
avgrej_BH = mean(rej_BH, na.rm = T)
avgrej_bonf = mean(rej_bonf, na.rm = T)
avgrej_holm = mean(rej_holm, na.rm = T)
avgrej_RW = mean(rej_RW, na.rm = T)

avgrej_knockoff_default = mean(rej_knockoff_default, na.rm = T)
avgrej_knockoff_fixed = mean(rej_knockoff_fixed, na.rm = T)
avgrej_knockoff_gaussian = mean(rej_knockoff_gaussian, na.rm = T)

# Only repetitions with non-zero output
avgrej2_knockoff_default = mean(rej_knockoff_default[rej_knockoff_default != 0], na.rm = T)
avgrej2_knockoff_fixed = mean(rej_knockoff_fixed[rej_knockoff_fixed != 0], na.rm = T)
avgrej2_knockoff_gaussian = mean(rej_knockoff_gaussian[rej_knockoff_gaussian != 0], na.rm = T)


# number of correct rejections

# beta_0 (identical in all repetitions)
betas <- res[[1]]$beta_0
names(betas) <- paste0("Var", 1:length(betas))

avgcorrej_RW = mean(vapply(res, function(x) sum(x$pvals_RW[betas!=0] <= alpha), double(1L)), na.rm = T)
avgcorrej_BH = mean(vapply(res, function(x) sum(x$pvals_BH[betas!=0] <= alpha), double(1L)), na.rm = T)
avgcorrej_bonf = mean(vapply(res, function(x) sum(x$pvals_bonf[betas!=0] <= alpha), double(1L)), na.rm = T)
avgcorrej_holm = mean(vapply(res, function(x) sum(x$pvals_holm[betas!=0] <= alpha), double(1L)), na.rm = T)
avgcorrej_none = mean(vapply(res, function(x) sum(x$pvals_none[betas!=0] <= alpha), double(1L)), na.rm = T)

sdcorrej_RW = sd(vapply(res, function(x) sum(x$pvals_RW[betas!=0] <= alpha), double(1L)), na.rm = T)
sdcorrej_BH = sd(vapply(res, function(x) sum(x$pvals_BH[betas!=0] <= alpha), double(1L)), na.rm = T)
sdcorrej_bonf = sd(vapply(res, function(x) sum(x$pvals_bonf[betas!=0] <= alpha), double(1L)), na.rm = T)
sdcorrej_holm = sd(vapply(res, function(x) sum(x$pvals_holm[betas!=0] <= alpha), double(1L)), na.rm = T)
sdcorrej_none = sd(vapply(res, function(x) sum(x$pvals_none[betas!=0] <= alpha), double(1L)), na.rm = T)

# knockoffs (via names)
# all repetitions
avgcorrej_knockoff_default = mean(vapply(res, function(x)  sum(names(x$knockoff_default$selected) %in% names(betas[betas!=0])), double(1L)), na.rm = T)
avgcorrej_knockoff_fixed = mean(vapply(res, function(x)  sum(names(x$knockoff_fixed$selected) %in% names(betas[betas!=0])), double(1L)), na.rm = T)
avgcorrej_knockoff_gaussian = mean(vapply(res, function(x)  sum(names(x$knockoff_gaussian$selected) %in% names(betas[betas!=0])), double(1L)), na.rm = T)

sdcorrej_knockoff_default = sd(vapply(res, function(x)  sum(names(x$knockoff_default$selected) %in% names(betas[betas!=0])), double(1L)), na.rm = T)
sdcorrej_knockoff_fixed = sd(vapply(res, function(x)  sum(names(x$knockoff_fixed$selected) %in% names(betas[betas!=0])), double(1L)), na.rm = T)
sdcorrej_knockoff_gaussian = sd(vapply(res, function(x)  sum(names(x$knockoff_gaussian$selected) %in% names(betas[betas!=0])), double(1L)), na.rm = T)

# only rejections with non-zero selections
avgcorrej2_knockoff_default = mean(vapply(res, function(x)  sum(names(x$knockoff_default$selected) %in% names(betas[betas!=0])), double(1L))[rej_knockoff_default != 0], na.rm = T)
avgcorrej2_knockoff_fixed = mean(vapply(res, function(x)  sum(names(x$knockoff_fixed$selected) %in% names(betas[betas!=0])), double(1L))[rej_knockoff_fixed != 0], na.rm = T)
avgcorrej2_knockoff_gaussian = mean(vapply(res, function(x)  sum(names(x$knockoff_gaussian$selected) %in% names(betas[betas!=0])), double(1L))[rej_knockoff_gaussian != 0], na.rm = T)

sdcorrej2_knockoff_default = sd(vapply(res, function(x)  sum(names(x$knockoff_default$selected) %in% names(betas[betas!=0])), double(1L))[rej_knockoff_default != 0], na.rm = T)
sdcorrej2_knockoff_fixed = sd(vapply(res, function(x)  sum(names(x$knockoff_fixed$selected) %in% names(betas[betas!=0])), double(1L))[rej_knockoff_fixed != 0], na.rm = T)
sdcorrej2_knockoff_gaussian = sd(vapply(res, function(x)  sum(names(x$knockoff_gaussian$selected) %in% names(betas[betas!=0])), double(1L))[rej_knockoff_gaussian != 0], na.rm = T)


# number of incorrect rejections
avgincorrej_RW = mean(vapply(res, function(x) sum(x$pvals_RW[betas==0] <= alpha), double(1L)), na.rm = T)
avgincorrej_BH = mean(vapply(res, function(x) sum(x$pvals_BH[betas==0] <= alpha), double(1L)), na.rm = T)
avgincorrej_bonf = mean(vapply(res, function(x) sum(x$pvals_bonf[betas==0] <= alpha), double(1L)), na.rm = T)
avgincorrej_holm = mean(vapply(res, function(x) sum(x$pvals_holm[betas==0] <= alpha), double(1L)), na.rm = T)
avgincorrej_none = mean(vapply(res, function(x) sum(x$pvals_none[betas==0] <= alpha), double(1L)), na.rm = T)

sdincorrej_RW = sd(vapply(res, function(x) sum(x$pvals_RW[betas==0] <= alpha), double(1L)), na.rm = T)
sdincorrej_BH = sd(vapply(res, function(x) sum(x$pvals_BH[betas==0] <= alpha), double(1L)), na.rm = T)
sdincorrej_bonf = sd(vapply(res, function(x) sum(x$pvals_bonf[betas==0] <= alpha), double(1L)), na.rm = T)
sdincorrej_holm = sd(vapply(res, function(x) sum(x$pvals_holm[betas==0] <= alpha), double(1L)), na.rm = T)
sdincorrej_none = sd(vapply(res, function(x) sum(x$pvals_none[betas==0] <= alpha), double(1L)), na.rm = T)

# knockoffs (via names)
# all repetitions
avgincorrej_knockoff_default = mean(vapply(res, function(x)  sum(names(x$knockoff_default$selected) %in% names(betas[betas==0])), double(1L)), na.rm = T)
avgincorrej_knockoff_fixed = mean(vapply(res, function(x)  sum(names(x$knockoff_fixed$selected) %in% names(betas[betas==0])), double(1L)), na.rm = T)
avgincorrej_knockoff_gaussian = mean(vapply(res, function(x)  sum(names(x$knockoff_gaussian$selected) %in% names(betas[betas==0])), double(1L)), na.rm = T)

sdincorrej_knockoff_default = sd(vapply(res, function(x)  sum(names(x$knockoff_default$selected) %in% names(betas[betas==0])), double(1L)), na.rm = T)
sdincorrej_knockoff_fixed = sd(vapply(res, function(x)  sum(names(x$knockoff_fixed$selected) %in% names(betas[betas==0])), double(1L)), na.rm = T)
sdincorrej_knockoff_gaussian = sd(vapply(res, function(x)  sum(names(x$knockoff_gaussian$selected) %in% names(betas[betas==0])), double(1L)), na.rm = T)

# only repetitions with non-zero selection
avgincorrej2_knockoff_default = mean(vapply(res, function(x)  sum(names(x$knockoff_default$selected) %in% names(betas[betas==0])), double(1L))[rej_knockoff_default != 0], na.rm = T)
avgincorrej2_knockoff_fixed = mean(vapply(res, function(x)  sum(names(x$knockoff_fixed$selected) %in% names(betas[betas==0])), double(1L))[rej_knockoff_fixed != 0], na.rm = T)
avgincorrej2_knockoff_gaussian = mean(vapply(res, function(x)  sum(names(x$knockoff_gaussian$selected) %in% names(betas[betas==0])), double(1L))[rej_knockoff_gaussian != 0], na.rm = T)

sdincorrej2_knockoff_default = sd(vapply(res, function(x)  sum(names(x$knockoff_default$selected) %in% names(betas[betas==0])), double(1L))[rej_knockoff_default != 0], na.rm = T)
sdincorrej2_knockoff_fixed = sd(vapply(res, function(x)  sum(names(x$knockoff_fixed$selected) %in% names(betas[betas==0])), double(1L))[rej_knockoff_fixed != 0], na.rm = T)
sdincorrej2_knockoff_gaussian = sd(vapply(res, function(x)  sum(names(x$knockoff_gaussian$selected) %in% names(betas[betas==0])), double(1L))[rej_knockoff_gaussian != 0], na.rm = T)


# FWER
FWER_RW = mean(vapply(res, function(x) any(x$pvals_RW[betas==0] <= alpha), double(1L)), na.rm = T)
FWER_BH = mean(vapply(res, function(x) any(x$pvals_BH[betas==0] <= alpha), double(1L)), na.rm = T)
FWER_bonf = mean(vapply(res, function(x) any(x$pvals_bonf[betas==0] <= alpha), double(1L)), na.rm = T)
FWER_holm = mean(vapply(res, function(x) any(x$pvals_holm[betas==0] <= alpha), double(1L)), na.rm = T)
FWER_none = mean(vapply(res, function(x) any(x$pvals_none[betas==0] <= alpha), double(1L)), na.rm = T)

# all repetitions 
FWER_knockoff_default = mean(vapply(res, function(x)  any(names(x$knockoff_default$selected) %in% names(betas[betas==0])), double(1L)), na.rm = T)
FWER_knockoff_fixed = mean(vapply(res, function(x)  any(names(x$knockoff_fixed$selected) %in% names(betas[betas==0])), double(1L)), na.rm = T)
FWER_knockoff_gaussian = mean(vapply(res, function(x)  any(names(x$knockoff_gaussian$selected) %in% names(betas[betas==0])), double(1L)), na.rm = T)

# only reptitions with non-zero selection
FWER2_knockoff_default = mean(vapply(res, function(x)  any(names(x$knockoff_default$selected) %in% names(betas[betas==0])), double(1L))[rej_knockoff_default != 0], na.rm = T)
FWER2_knockoff_fixed = mean(vapply(res, function(x)  any(names(x$knockoff_fixed$selected) %in% names(betas[betas==0])), double(1L))[rej_knockoff_fixed != 0], na.rm = T)
FWER2_knockoff_gaussian = mean(vapply(res, function(x)  any(names(x$knockoff_gaussian$selected) %in% names(betas[betas==0])), double(1L))[rej_knockoff_gaussian != 0], na.rm = T)

# FDR: Proportion of false among all rejections
FDR_RW = mean( vapply(res, function(x) sum(x$pvals_RW[betas==0] <= alpha), double(1L))/vapply(res, function(x) sum(x$pvals_RW <= alpha), double(1L)), na.rm = T)
FDR_BH = mean( vapply(res, function(x) sum(x$pvals_BH[betas==0] <= alpha), double(1L))/vapply(res, function(x) sum(x$pvals_BH <= alpha), double(1L)), na.rm = T)
FDR_bonf = mean( vapply(res, function(x) sum(x$pvals_bonf[betas==0] <= alpha), double(1L))/vapply(res, function(x) sum(x$pvals_bonf <= alpha), double(1L)), na.rm = T)
FDR_holm = mean( vapply(res, function(x) sum(x$pvals_holm[betas==0] <= alpha), double(1L))/vapply(res, function(x) sum(x$pvals_holm <= alpha), double(1L)), na.rm = T)
FDR_none = mean( vapply(res, function(x) sum(x$pvals_none[betas==0] <= alpha), double(1L))/vapply(res, function(x) sum(x$pvals_none <= alpha), double(1L)), na.rm = T)

# all repetitions (use reps with zero-selection as zero rejection rounds)
FDR_knockoff_default =  mean(replace_na(vapply(res, function(x)  sum(names(x$knockoff_default$selected) %in% names(betas[betas==0])), double(1L)) / rej_knockoff_default, 0), na.rm = T)
FDR_knockoff_fixed = mean(replace_na(vapply(res, function(x)  sum(names(x$knockoff_fixed$selected) %in% names(betas[betas==0])), double(1L)) / rej_knockoff_fixed, 0), na.rm = T)
FDR_knockoff_gaussian =  mean(replace_na(vapply(res, function(x)  sum(names(x$knockoff_gaussian$selected) %in% names(betas[betas==0])), double(1L)) / rej_knockoff_gaussian, 0), na.rm = T)

# only repetitions with non-zero selection
FDR2_knockoff_default = 
 mean(vapply(res, function(x)  sum(names(x$knockoff_default$selected) %in% names(betas[betas==0])), double(1L))[ rej_knockoff_default != 0] / rej_knockoff_default[ rej_knockoff_default != 0], na.rm = T)
FDR2_knockoff_fixed = 
 mean(vapply(res, function(x)  sum(names(x$knockoff_fixed$selected) %in% names(betas[betas==0])), double(1L)) / rej_knockoff_fixed, na.rm = T)
FDR2_knockoff_gaussian = 
 mean(vapply(res, function(x)  sum(names(x$knockoff_gaussian$selected) %in% names(betas[betas==0])), double(1L)) / rej_knockoff_gaussian, na.rm = T)

# compute MSE for coefficients (rlassoEffects vs. knockoffs)




#### Result table
results <- matrix( NA, ncol = 8, nrow = 12, byrow = FALSE)

# avg correct rejections
results[1, ] <- c(avgcorrej_none, avgcorrej_BH, avgcorrej_bonf, avgcorrej_holm, avgcorrej_RW, avgcorrej_knockoff_default, avgcorrej_knockoff_fixed, avgcorrej_knockoff_gaussian)

# sd correct rejection
results[2, ] <- c(sdcorrej_none, sdcorrej_BH, sdcorrej_bonf, sdcorrej_holm, sdcorrej_RW, sdcorrej_knockoff_default, sdcorrej_knockoff_fixed, sdcorrej_knockoff_gaussian)

# avg correct rejections (knockoff only with non-zero selection)
results[3, 6:8] <- c(avgcorrej2_knockoff_default, avgcorrej2_knockoff_fixed, avgcorrej2_knockoff_gaussian)

# avg correct rejections (knockoff only with non-zero selection)
results[4, 6:8] <- c(sdcorrej2_knockoff_default, sdcorrej2_knockoff_fixed, sdcorrej2_knockoff_gaussian)

# avg incorrect rejections
results[5, ] <- c(avgincorrej_none, avgincorrej_BH, avgincorrej_bonf, avgincorrej_holm, avgincorrej_RW, avgincorrej_knockoff_default, avgincorrej_knockoff_fixed, avgincorrej_knockoff_gaussian)

# sd incorrect rejections 
results[6, ] <- c(sdincorrej_none, sdincorrej_BH, sdincorrej_bonf, sdincorrej_holm, sdincorrej_RW, sdincorrej_knockoff_default, sdincorrej_knockoff_fixed, sdincorrej_knockoff_gaussian)

# avg incorrect rejections (knockoff only with non-zero selection)
results[7, 6:8] <- c(avgincorrej2_knockoff_default, avgincorrej2_knockoff_fixed, avgincorrej2_knockoff_gaussian)

# avg incorrect rejections (knockoff only with non-zero selection)
results[8, 6:8] <- c(sdincorrej2_knockoff_default, sdincorrej2_knockoff_fixed, sdincorrej2_knockoff_gaussian)

# FWER
results[9, ] <- c(FWER_none, FWER_BH, FWER_bonf, FWER_holm, FWER_RW, FWER_knockoff_default, FWER_knockoff_fixed, FWER_knockoff_gaussian)

# FWER (knockoff only with non-zero selection)
results[10, 6:8] <- c(FWER2_knockoff_default, FWER2_knockoff_fixed, FWER2_knockoff_gaussian)

# FDR 
results[11, ] <- c(FDR_none, FDR_BH, FDR_bonf, FDR_holm, FDR_RW, FDR_knockoff_default, FDR_knockoff_fixed, FDR_knockoff_gaussian)

# FDR (knockoff only with non-zero selection)
results[12, 6:8] <- c(FDR2_knockoff_default, FDR2_knockoff_fixed, FDR2_knockoff_gaussian)

colnames(results) = c("none", "BH", "bonf", "holm", "RW", "kn.off default", "kn.off fixed", "kn.off X")
rownames(results) <- c("Correct", "(sd)", "Correct2", "(sd)", "Incorrect", "(sd)", "Incorrect2", "(sd)", "FWER", "FWER2" , "FDR", "FDR2")


save(results, file = paste0(wspace, "/results/", date, "_result_merge_medium_n_", n, "_p_", p, "_R_", R, ".RData" ))



# knockoff: Number of failed / succeeded repetitions
fail_knockoff_default = sum(vapply(res, function(x) length(x$knockoff_default$selected) == 0, double(1L)))
success_knockoff_default = R - fail_knockoff_default
fail_knockoff_fixed = sum(vapply(res, function(x) length(x$knockoff_fixed$selected) == 0, double(1L)))
success_knockoff_fixed = R - fail_knockoff_fixed
fail_knockoff_gaussian = sum(vapply(res, function(x) length(x$knockoff_gaussian$selected) == 0, double(1L)))
success_knockoff_gaussian = R - fail_knockoff_gaussian


# reason for failure
threshold_knockoff_default = vapply(res, function(x) x$knockoff_default$threshold, double(1L))
threshold_knockoff_fixed = vapply(res, function(x) x$knockoff_fixed$threshold, double(1L))
threshold_knockoff_gaussian = vapply(res, function(x) x$knockoff_gaussian$threshold, double(1L))


# Fail, success
knockoff_fail = list("fail_default" = fail_knockoff_default, 
                     "fail_fixed" = fail_knockoff_fixed, 
                     "fail_gaussian" = fail_knockoff_gaussian,
                     "success_default" = success_knockoff_default,
                     "success_fixed" = success_knockoff_fixed, 
                     "success_gaussian" = success_knockoff_gaussian)

save(knockoff_fail, file = paste0(wspace, "/results/", date, "_fail_knockoff_medium_n_", n, "_p_", p, "_R_", R, ".RData" ))

